package Crypt::Perl::Ed25519;

use strict;
use warnings;

=encoding utf-8

=head1 NAME

Crypt::Perl::Ed25519 - Ed25519 in pure Perl

=head1 DESCRIPTION

This module contains no code; see one of:

=over

=item * L<Crypt::Perl::Ed25519::PrivateKey>

=item * L<Crypt::Perl::Ed25519::PublicKey>

=item * L<Crypt::Perl::Ed25519::Parse>

=back

Crypt::Perl’s Ed25519 interface is EXPERIMENTAL. The underlying logic is ported
from L<https://github.com/digitalbazaar/forge/blob/master/lib/ed25519.js>.

See L<RFC 8032|https://tools.ietf.org/html/rfc8032> for the gory details of
Ed25519.

=cut

1;
